<?php
/**
 * Speedcache
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 * @package Speedcache
 * @copyright Copyright (C) 2016 JoomUnited (https://www.joomunited.com). All rights reserved.
 * @license GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */


defined('_JEXEC') or die;

/**
 * speedcache Component speedcache
 *
 * @since  1.6
 */
class speedcacheControllerConfiguration extends JControllerForm
{
    /**
     * Constructor
     *
     * @throws Exception
     */
    public function __construct()
    {
        parent::__construct();
    }

    public function display($cachable = false, $urlparams = false)
    {
        // Force it to be the search view
        $this->input->set('view', 'configuration');
        parent::display();
    }

    public function save_params()
    {
        //Security check
        JSession::checkToken() or die('Invalid Token');

        $result = $this->save_to_configuration();

        $type = 'success';
        $text = JText::_('COM_SPEEDCACHE_CONFIGURATION_MESSAGE_SAVE_SUCCESS');

        if (!$result) {
            $type = 'error';
            $text = $this->getError();
        }

        JFactory::getSession()->set(
            'sc_configuration_message',
            array(
                'type' => $type,
                'text' => $text
            )
        );

        $this->cleanCaches();

        $this->setRedirect(JRoute::_('index.php?option=com_speedcache&task=configuration.display', false));
    }

    public function save_params_and_close()
    {
        //Security check
        JSession::checkToken() or die('Invalid Token');

        $result = $this->save_to_configuration();

        if (!$result) {
            $this->setMessage($this->getError(), 'error');
        } else {
            $this->setMessage(JText::_('COM_SPEEDCACHE_CONFIGURATION_MESSAGE_SAVE_SUCCESS'), 'message');
        }

        $this->cleanCaches();

        $this->setRedirect(JRoute::_('index.php?option=com_speedcache', false));
    }

    public function save_to_configuration()
    {
        $input = JFactory::getApplication()->input;
        // If we are on the save command we need the actual data
        $jformData = $input->get('jform', array(), 'array');

        if (empty($jformData)) {
            $this->setError(JText::_('COM_SPEEDCACHE_CONFIGURATION_NO_GET_FORM_DATA_ERROR'));
            return false;
        }

        $model = $this->getModel();
        $global = $model->saveGlobalConfig($jformData);
        $permissions = $model->savePermissions($jformData);

        if (!$global) {
            $this->setError(JText::_('COM_SPEEDCACHE_CONFIGURATION_MESSAGE_SAVE_GLOBAL_CONFIG_ERROR'));
            return false;
        }

        if (!$permissions) {
            return false;
        }

        return true;
    }

    public function close_configuration()
    {
        $this->setRedirect(JRoute::_('index.php?option=com_speedcache', false));
    }

    /**
     * Clean admin and front system cache
     */
    protected function cleanCaches()
    {
        //Clean user state
        JFactory::getApplication()->setUserState('com_config.config.global.data', null);
        JModelLegacy::addIncludePath(JPATH_ROOT . '/administrator/components/com_cache/models');
        $model = JModelLegacy::getInstance('cache', 'CacheModel');

        if (!empty($model)) {
            $clients = array(1, 0);
            foreach ($clients as $client) {
                $mCache = $model->getCache($client);
                foreach ($mCache->getAll() as $cache) {
                    $mCache->clean($cache->group);
                }
            }
        }
    }
}
