<?php
/**
 * Speedcache
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 * @package Speedcache
 * @copyright Copyright (C) 2016 JoomUnited (https://www.joomunited.com). All rights reserved.
 * @license GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */


defined('_JEXEC') or die;

/**
 * speedcache Component speedcache
 *
 * @since  1.6
 */
class speedcacheControllerWizard extends JControllerLegacy
{
    /**
     * Constructor
     *
     * @throws Exception
     */
    public function __construct()
    {
        parent::__construct();
    }

    public function configuration_environment()
    {
        $this->change_steps('environment', 0);
    }

    public function configuration_quick()
    {
        $this->change_steps('quick', 1);
    }

    public function configuration_advanced()
    {
        $this->change_steps('advanced', 2);
    }

    public function configuration_minify()
    {
        //Security check
        JSession::checkToken() or die('Invalid Token');

        $this->setParameterSettings();
        // Redirect
        $this->change_steps('minify', 3);
    }

    public function configuration_done()
    {
        //Security check
        JSession::checkToken() or die('Invalid Token');

        $this->setMinifySettings();

        $view = $this->getView('wizard', JFactory::getDocument()->getType());
        $view->setLayout('done');
        // Display the view
        $view->display();

        return $this;
    }

    public function automatic_configuration_quick()
    {
        //Security check
        JSession::checkToken() or die('Invalid Token');

        $this->setDefaultSettings();
        // Redirect to the edit screen.
        $this->setRedirect(JRoute::_('index.php?option=com_speedcache', false));
        return true;
    }

    public function change_steps($layout, $tab)
    {
        //Security check
        JSession::checkToken() or die('Invalid Token');

        JRequest::setVar('configuration_template', $layout, 'POST');
        JRequest::setVar('current_tab', $tab, 'POST');

        $view = $this->getView('wizard', JFactory::getDocument()->getType());
        $view->setLayout('configuration');
        // Display the view
        $view->display();

        return $this;
    }

    public function setParameterSettings()
    {
        $settings = array(
            'use_browser_cache' => JFactory::getApplication()->input->getInt('use_browser_cache', 0),
            'cache_lifetime' => JFactory::getApplication()->input->getInt('cache_lifetime', 1440),
            'distinct_user_cache' => JFactory::getApplication()->input->getInt('distinct_user_cache', 0),
            'cache_header' => JFactory::getApplication()->input->getInt('cache_header', 0),
            'auto_add_url' => JFactory::getApplication()->input->getInt('auto_add_url', 0),
            'cache_external_script' => JFactory::getApplication()->input->getInt('cache_external_script', 0)
        );
        //Set configuration
        speedcacheComponentHelper::setParams($settings);
    }

    public function setMinifySettings()
    {
        $settings = array(
            'minify_html' => JFactory::getApplication()->input->getInt('minify_html', 0),
            'defer_css' => JFactory::getApplication()->input->getInt('defer_css', 0),
            'defer_js' => JFactory::getApplication()->input->getInt('defer_js', 0),
            'minify_group_css' => JFactory::getApplication()->input->getInt('minify_group_css', 0),
            'minify_group_js_inhead' => JFactory::getApplication()->input->getInt('minify_group_js_inhead', 0),
            'minify_group_js' => JFactory::getApplication()->input->getInt('minify_group_js', 0),
        );
        //Set configuration
        speedcacheComponentHelper::setParams($settings);
    }

    public function setDefaultSettings()
    {
        $settings = array(
            'use_browser_cache' => 1,
            'cache_lifetime' => 1440,
            'distinct_user_cache' => 1,
            'cache_header' => 1,
            'auto_add_url' => 1,
            'cache_external_script' => 0,
            'minify_html' => 1,
            'defer_css' => 0,
            'defer_js' => 0,
            'minify_group_css' => 1,
            'minify_group_js_inhead' => 1,
            'minify_group_js' => 0,
        );
        //Set configuration
        if (speedcacheComponentHelper::setParams($settings)) {
            $this->setMessage(JText::_('COM_SPEEDCACHE_WIZARD_AUTOMATIC_SET_QUICK_SUCCESS'), 'success');
        } else {
            $this->setMessage(JText::_('COM_SPEEDCACHE_WIZARD_AUTOMATIC_SET_QUICK_ERROR'), 'error');
        }

        return true;
    }

    public function check_apache_modules()
    {
        //Check token
        if (!JSession::checkToken('GET')) {
            echo json_encode(array('status' => 'error'));
            jexit();
        }

        $modules = array(
            'mod_expires' => false,
            'gzip' => false,
            'mod_headers' => false,
        );

        $modules['gzip'] = $this->checkGzip();
        $modules['mod_expires'] = $this->checkApacheModuleExists('mod_expires');
        $modules['mod_headers'] = $this->checkApacheModuleExists('mod_headers');

        echo json_encode($modules);
        jexit();
    }

    /**
     * Call the front home page to retrieve headers and
     * check if it contains the encoding gzip
     * @return bool
     */
    private function checkGzip()
    {
        $ch = curl_init(JUri::root());
        curl_setopt($ch, CURLOPT_ENCODING, "gzip");
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HEADER, true);
        $response = curl_exec($ch);
        $info = curl_getinfo($ch);

        $headers = substr($response, 0, $info['header_size']);

        foreach (explode("\r\n", $headers) as $header) {
            $headerContent = explode(':', $header);

            if (trim($headerContent[0]) === 'Content-Encoding' || trim($headerContent[0]) === 'content-encoding') {
                return true;
            }
        }

        return false;
    }

    /**
     * Check module exist in apache
     * @param $module_name
     * @return bool
     */
    private function checkApacheModuleExists($module_name)
    {
        if (function_exists('apache_get_modules')) {
            $modules = apache_get_modules();
            return (in_array($module_name, $modules) ? true : false);
        } else {
            //Files to check
            $moduleOK = false;
            $ch = curl_init(JUri::root());
            curl_setopt($ch, CURLOPT_ENCODING, "gzip");
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_HEADER, true);
            $response = curl_exec($ch);
            $info = curl_getinfo($ch);

            $headers = substr($response, 0, $info['header_size']);
            foreach (explode("\r\n", $headers) as $header) {
                $headerContent = explode(':', $header);
                if ($module_name === 'mod_expires' && trim($headerContent[0]) === 'Expires' && !empty($headerContent[1])) {
                    $moduleOK = true;
                    break;
                }
            }

            return (($moduleOK) ? true : false);
        }
    }

}
