<?php
/**
 * @package     Joomla.Administrator
 * @subpackage  com_config
 *
 * @copyright   Copyright (C) 2005 - 2020 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

/**
 * Model for component configuration
 *
 * @since  3.2
 */
class speedcacheModelConfiguration extends JModelAdmin
{
    /**
     * Method to get form file config
     *
     * @param array $data File data
     * @param boolean $loadData Load data
     *
     * @return boolean
     * @since  version
     */
    public function getForm($data = array(), $loadData = true)
    {
        // Add the search path for the admin component config.xml file.
        JForm::addFormPath(JPATH_ADMINISTRATOR . '/components/com_speedcache');

        $form = $this->loadForm('com_speedcache.config', 'config', array('control' => 'jform', 'load_data' => $loadData), false, '/config');

        if (empty($form)) {
            return false;
        }

        return $form;
    }

    public function saveGlobalConfig($jformData)
    {
        //Load the data form
        $form = $this->getForm();

        $default_parameter = array();
        foreach ($form->getFieldsets() as $name => $fieldsets) {
            foreach ($form->getFieldset($name) as $fieldset) {
                if ($fieldset->type === 'Radio') {
                    $fieldset->value = 0;
                }
                $default_parameter[$fieldset->fieldname] = $fieldset->value;
            }
        }

        $new_parameter = array_replace($default_parameter, $jformData);


        //Set configuration
        if (!speedcacheComponentHelper::setParams($new_parameter)) {
            return false;
        }

        return true;
    }

    public function savePermissions($jformData)
    {
        // Save the rules.
        if (isset ($jformData) && isset ($jformData['rules'])) {
            $form = $this->getForm($jformData);

            // Validate the posted data.
            $postedRules = $this->validate($form, $jformData);

            $rules = new JAccessRules ($postedRules['rules']);
            $asset = JTable::getInstance('asset');

            if (!$asset->loadByName($jformData['component'])) {
                $root = JTable::getInstance('asset');
                $root->loadByName('root.1');
                $asset->name = $jformData['component'];
                $asset->title = $jformData['component'];
                $asset->setLocation($root->id, 'last-child');
            }
            $asset->rules = (string)$rules;

            if (!$asset->check() || !$asset->store()) {
                $this->setError($asset->getError());
                return false;
            }
        }

        return true;
    }
}
