<?php
/**
 * Speedcache
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 * @package Speedcache
 * @copyright Copyright (C) 2016 JoomUnited (https://www.joomunited.com). All rights reserved.
 * @license GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

defined('_JEXEC') or die;

/**
 * View class for a list of speedcache.
 *
 * @since  1.6
 */
class speedcacheViewUrls extends JViewLegacy
{
    protected $items;

    protected $cacheitems;

    protected $pagination;

    protected $state;

    protected $user;

    protected $listOrder;

    protected $listDirn;
    /**
     * Execute and display a template script.
     *
     * @param   string $tpl The name of the template file to parse; automatically searches through the template paths.
     *
     * @return  mixed  A string if successful, otherwise a Error object.
     *
     * @since   1.6
     */
    public function display($tpl = null)
    {
        // Include the component HTML helpers.
        JHtml::addIncludePath(JPATH_COMPONENT . '/helpers/html');
        JHtml::_('bootstrap.tooltip');
        JHtml::_('behavior.multiselect');

        JHtml::_('jquery.framework');
        JHTML::_('behavior.modal');

        JFactory::getDocument()->addScript(JUri::base() . 'components/com_speedcache/assets/js/urls.js');

        $this->items = $this->get('Items');
        $this->cacheitems = $this->get('CacheItems');
        $this->pagination = $this->get('Pagination');
        $this->state = $this->get('State');

        $this->user = JFactory::getUser();
        $this->listOrder = $this->escape($this->state->get('list.ordering'));
        $this->listDirn = $this->escape($this->state->get('list.direction'));

        // Check for errors.
        if (count($errors = $this->get('Errors'))) {
            JError::raiseError(500, implode("\n", $errors));
            return false;
        }

        $this->addToolbar();

        parent::display($tpl);
    }

    /**
     * Add the page title and toolbar.
     *
     * @return  void
     *
     * @since   1.6
     */
    protected function addToolbar()
    {
        $canDo = JHelperContent::getActions('com_speedcache');
        JToolbarHelper::title(JText::_('COM_SPEEDCACHE_MANAGER'));
        $bar = JToolbar::getInstance('toolbar');
        JHTML::_('behavior.tooltip');
        if ($canDo->get('core.create')) {
            JToolbarHelper::addNew('url.add');

            $dhtml = "<button class='ju-button orange-button'>
			<span class='material-icons'>vertical_align_bottom</span>" . JText::_('COM_SPEEDCACHE_BTN_ADD_FROM_MENU') . "</button>";

            $bar->appendButton('Custom', $dhtml, "speedcacheimportmenubtn");

            $bar->appendButton('Standard','addincludeRules',JText::_('COM_SPEEDCACHE_BTN_ADD_NEW'),'url.addincludeRulesItems',false);

            $bar->appendButton('Standard','addexclude',JText::_('COM_SPEEDCACHE_BTN_ADD_NEW'),'url.addexcludeItems',false);

            $bar->appendButton('Standard','addexcludeRules',JText::_('COM_SPEEDCACHE_BTN_ADD_NEW'),'url.addexcludeRulesItems',false);
        }

        if ($canDo->get('core.create')) {
            $dhtml = "<button class='ju-button orange-button'>
			<span class='material-icons'>vertical_align_bottom</span>" . JText::_('COM_SPEEDCACHE_BTN_SELECT_FROM_MENU') . "</button>";

            $bar->appendButton('Custom', $dhtml, "speedcacheselectmenubtn");
        }

        if ($canDo->get('core.edit.state')) {
            JToolbarHelper::divider();

            //Select batch process type
            $dhtml = '<select name="action_state">
				<option value="guest">' . JText::_('COM_SPEEDCACHE_BULK_CHANGE_CACHE_GUEST') . '</option>
				<option value="preloadguest">' . JText::_('COM_SPEEDCACHE_BULK_CHANGE_PRELOAD_GUEST') . '</option>
				<option value="logged">' . JText::_('COM_SPEEDCACHE_BULK_CHANGE_CACHE_LOGGED') . '</option>
				<option value="preloadlogged">' . JText::_('COM_SPEEDCACHE_BULK_CHANGE_PRELOAD_LOGGED') . '</option>
				<option value="preloaduser">' . JText::_('COM_SPEEDCACHE_BULK_CHANGE_PRELOAD_USER') . '</option>
				<option value="excludeguest">' . JText::_('COM_SPEEDCACHE_BULK_CHANGE_EXCLUDE_GUEST') . '</option>
				<option value="excludelogged">' . JText::_('COM_SPEEDCACHE_BULK_CHANGE_EXCLUDE_LOGGED') . '</option>
			</select>';
            $bar->appendButton('Custom', $dhtml, "batch");

            $topstandard_toolbar = array(
                'guest' => 'urls.cacheGuest',
                'unguest' => 'urls.uncacheGuest',
                'preloadguest' => 'urls.preloadGuest',
                'unpreloadguest' => 'urls.unpreloadGuest',
                'logged' => 'urls.cacheLogged',
                'unlogged' => 'urls.uncacheLogged',
                'preloadlogged' => 'urls.preloadLogged',
                'unpreloadlogged' => 'urls.unpreloadLogged',
                'preloaduser' => 'urls.preloadperuser',
                'unpreloaduser' => 'urls.unpreloadperuser',
                'excludeguest' => 'urls.excludeGuest',
                'unexcludeguest' => 'urls.unexcludeGuest',
                'excludelogged' => 'urls.excludeLogged',
                'unexcludelogged' => 'urls.unexcludeLogged');

            foreach ($topstandard_toolbar as $k => $task) {
                $tooltip = JText::_('COM_SPEEDCACHE_BULK_CHANGE_TOPSTANDARD_ENABLE');
                if (strpos($k, 'un') !== false) {
                    $tooltip = JText::_('COM_SPEEDCACHE_BULK_CHANGE_TOPSTANDARD_DISABLE');
                }

                $tpbar = "<button class='btn button-".$k." hasTip' title='".$tooltip."'
                        onclick=\"if (document.adminForm.boxchecked.value == 0) {
                                    alert(Joomla.JText._('JLIB_HTML_PLEASE_MAKE_A_SELECTION_FROM_THE_LIST'));
                                  } else {
                                   Joomla.submitbutton('$task'); }\" >
                                   <span class='material-icons'></span>
                       </button>";
                $bar->appendButton('Custom', $tpbar, $k);
            }
        }

        JToolbarHelper::divider();

        if ($canDo->get('core.delete')) {
            JToolbarHelper::divider();
            $delete_quest = JText::_('JGLOBAL_CONFIRM_DELETE');
            $deletebar = "<button class='btn button-delete hasTip' title='".JText::_('COM_SPEEDCACHE_BULK_CHANGE_TOPSTANDARD_DELETE')."'
                        onclick=\"if (document.adminForm.boxchecked.value == 0) {
                                    alert(Joomla.JText._('JLIB_HTML_PLEASE_MAKE_A_SELECTION_FROM_THE_LIST'));
                                  } else {
                                    var cf = confirm('$delete_quest');
                                  
                                    if (cf == true) {
                                     Joomla.submitbutton('urls.delete');
                                    } 
                                  } \" >
                                   <span class=\"icon-delete\" aria-hidden=\"true\"></span>
                       </button>";
            $bar->appendButton('Custom', $deletebar, 'delete');
        }

        if ($canDo->get('core.admin')) {
            $bohtml = "<button class='ju-button orange-outline-button orange-border waves-effect waves-light' onclick='Joomla.submitbutton(\"urls.redirectConfiguration\");'>
			<span class='material-icons-outlined'>settings</span> " . JText::_('COM_SPEEDCACHE_BTN_BACK_OPTIONS') . "</button>";
            $bar->appendButton('Custom', $bohtml, "speedcachebackoptions");

            $bdhtml = "<button class='ju-button orange-button waves-effect waves-light' onclick='Joomla.submitbutton(\"dashboard\");'>
			<span class='material-icons-outlined'>dashboard</span> " . JText::_('COM_SPEEDCACHE_BTN_BACK_DASHBOARD') . "</button>";
            $bar->appendButton('Custom', $bdhtml, "speedcachebackdashboard");
        }
    }
}
