<?php

/*
 * Using lazy-loading-xt jquery plugin
 */
include('image-process/image-resizer.php');
/**
 * Class SCLazyLoading
 */
class SCLazyLoading {
    private $content;
    private $list_exclude;
    private $disallow_width;
    private $disallow_height;
    /**
     * SCLazyLoading constructor.
     * @param $contents
     */
    public function __construct($contents, $list_exclude, $disallow_size)
    {
        $this->content = $contents;
        $this->list_exclude = $list_exclude;
        $this->disallow_width = (int)$disallow_size[0];
        $this->disallow_height = (int)$disallow_size[1];
    }

    /**
     * Set lazyloading image on contents
     * @return mixed
     */
    public function setup()
    {
        // Get img in script to array to check
        $imginscript = array();
        if (preg_match_all('#<script(.*?)>(.*?)</script>#is', $this->content, $matches)){
            foreach ($matches[0] as $scripttag) {
                if (preg_match_all('#<img\s[^>]*?src\s*=\s*[\'\"]([^\'\"]*?)[\'\"][^>]*?>#Usmi', $scripttag, $matches1)) {
                    foreach ($matches1[0] as $imgin) {
                        $imginscript[] = $imgin;
                    }
                }
            }
        }
        if (preg_match_all('#<img\s[^>]*?src\s*=\s*[\'\"]([^\'\"]*?)[\'\"][^>]*?>#Usmi', $this->content, $matches)) {
            $placeholder_url = 'data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAQAAAC1HAwCAAAAC0lEQVR42mP8/x8AAwMCAO+ip1sAAAAASUVORK5CYII=';

            foreach ($matches[0] as $imgtag) {
                //Cancel the img tag in the script
                if (in_array($imgtag, $imginscript)) {
                    continue;
                }

                if ($this->strposa($imgtag, $this->list_exclude)) {
                    continue;
                }

                if (!preg_match("/src=['\"]data:image/is", $imgtag)) {
                    // Low quality image placeholder
                    preg_match('/src=["|\'](.*?)["|\']/i', $imgtag, $image_match);
                    if (empty($image_match)) {
                        continue;
                    }
                    if (isset($image_match[1])) {
                        $fullLinkImage = $this->getFullLink($image_match[1]);
                        // Get size image
                        list($width_img, $height_img) = $this->getOriginalSize($fullLinkImage);

                        $check_size_to_lazyload = $this->checkAllowSizeImages($width_img, $height_img);
                        if (!$check_size_to_lazyload) {
                            continue;
                        }

                        $placeholder_url = $this->resizeImages($fullLinkImage);

                        if (!$placeholder_url) {
                            // If can not resize, get default placeholder
                            $placeholder_url = 'data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7';
                        }
                    }
                    // replace the src and add the data-src attribute
                    $imgtag_data = preg_replace(
                        '/<img(.*?)src=/is',
                        '<img$1src="' . $placeholder_url . '" data-speedcachelazy-src=',
                        $imgtag
                    );

                    // replace the srcset
                    $imgtag_data = str_replace('srcset', 'data-speedcachelazy-srcset', $imgtag_data);

                    // Add width and height attribute
                    if (!empty($width_img) && !preg_match('/(\s)+width=["|\'](.*?)["|\']/', $imgtag)) {
                        $imgtag_data = preg_replace(
                            '/<img/is',
                            '<img width="'.$width_img.'"',
                            $imgtag_data
                        );
                    }

                    if (!empty($height_img) && !preg_match('/(\s)+height=["|\'](.*?)["|\']/', $imgtag)) {
                        $imgtag_data = preg_replace(
                            '/<img/is',
                            '<img height="'.$height_img.'"',
                            $imgtag_data
                        );
                    }

                    // add the lazy class to the img element
                    if (preg_match('/class=["\']/i', $imgtag_data)) {
                        $imgtag_data = preg_replace(
                            '/class=(["\'])(.*?)["\']/is',
                            'class=$1speedcache-lazy speedcache-lazy-hidden $2$1',
                            $imgtag_data
                        );
                    } else {
                        $imgtag_data = preg_replace(
                            '/<img/is',
                            '<img class="speedcache-lazy speedcache-lazy-hidden"',
                            $imgtag_data
                        );
                    }
                    $noscript = "<noscript>" . $imgtag . "</noscript>";
                    $imgtag_data .= $noscript;

                    // Replace new img tag to old img tag
                    $this->content = str_replace($imgtag, $imgtag_data, $this->content);
                }
            }
        }
        return $this->content;
    }

    /**
     * Get original size of image
     *
     * @param string $link Link of imag
     *
     * @return array
     */
    public function getOriginalSize($link)
    {
        // Get image information
        $image_path_info = pathinfo($link);
        // Get local image
        $local_file = realpath($_SERVER['DOCUMENT_ROOT'] . parse_url($image_path_info['dirname'], PHP_URL_PATH));
        $file_name = '/' . $image_path_info['basename'];

        if (!$local_file || !file_exists($local_file . $file_name)) {
            // If it is an external image, not exist in the server (example : avatar image)
            // Or file was not found
            return null;
        }

        $size = getimagesize($local_file . $file_name);

        if (!$size) {
            return null;
        }

        return $size;
    }

    /**
     * Get full link of image
     *
     * @param string $link Link of imag
     *
     * @return string
     */
    public function getFullLink($link)
    {
        // Removing query string
        $link = preg_replace('/(\?|\#|\&).*/', '', $link);

        // Determine if the request was over SSL (HTTPS).
        if (isset($_SERVER['HTTPS']) && !empty($_SERVER['HTTPS']) && (strtolower($_SERVER['HTTPS']) != 'off'))
        {
            $http = 'https:';
        } elseif ((isset($_SERVER['HTTP_X_FORWARDED_PROTO']) &&
            !empty($_SERVER['HTTP_X_FORWARDED_PROTO']) &&
            (strtolower($_SERVER['HTTP_X_FORWARDED_PROTO']) !== 'http'))) {
            $http = 'https:';
        } else {
            $http = 'http:';
        }

        // Decode link
        if (strpos($link, '%') !== false) {
            $link = urldecode($link);
        }

        // Normalize
        if (strpos($link, '//') === 0) {
            $link = $http . $link;
        }

        // Adds 'http://' if none is set
        if (strpos($link, '/') === 0) {
            // Relative URL in component. use server http_host.
            $link = $http . '//' . $_SERVER['HTTP_HOST'] . $link;
        } elseif (strpos($link, 'http') === false && strpos($link, 'https') === false) {
            $link = $http . '//' . $link;
        }

        return $link;
    }

    /**
     * Convert image from original to low quality
     *
     * @param string $img_src Link of image to resize
     *
     * @return string|boolean
     */
    public function resizeImages($img_src)
    {
        // Get image information
        $image_path_info = pathinfo($img_src);
        // Get image parameter
        $source_file = realpath($_SERVER['DOCUMENT_ROOT'] . parse_url($image_path_info['dirname'], PHP_URL_PATH));
        $input_file = '/' . $image_path_info['basename'];
        $output_file = '/small_' . $image_path_info['basename'];

        if (!$source_file || !file_exists($source_file . $input_file)) {
            // If it is an external image, not exist in the server (example : avatar image)
            // Or file was not found
            return false;
        }

        // Execute resize image to low quality
        $resizer = new \SpeedCache\LazyLoading\ImageResizer(
            $source_file,
            $input_file,
            $source_file,
            $output_file,
            20,
            20,
            '',
            10
        );

        // An error occurred while retrieving image data
        if (!$resizer->validate()) {
            return false;
        }

        $imageContent = $resizer->output(false, false);
        if ($imageContent) {
            $type = pathinfo($output_file, PATHINFO_EXTENSION);
            $base64 = 'data:image/' . $type . ';base64,' . base64_encode($imageContent);

            return $base64;
        }

        return false;
    }

    /**
     * Check allow lazy loading for images
     *
     * @param integer $width_img  Width of imag
     * @param integer $height_img Height of imag
     *
     * @return boolean
     */
    public function checkAllowSizeImages($width_img, $height_img)
    {
        if (empty($width_img) && empty($height_img)) {
            return true;
        }

        if (empty($this->disallow_width) && empty($this->disallow_height)) {
            return true;
        }

        if (!empty($this->disallow_width) && empty($this->disallow_height)) {
            if ($width_img < $this->disallow_width) {
                return false;
            }
        }

        if (!empty($this->disallow_height) && empty($this->disallow_width)) {
            if ($height_img < $this->disallow_height) {
                return false;
            }
        }

        if (($width_img < $this->disallow_width) || ($height_img < $this->disallow_height)) {
            return false;
        }

        return true;
    }



    public function strposa($haystack, $needle, $offset=0) {
        if(!is_array($needle)) $needle = array($needle);

        foreach($needle as $query) {
            if (empty($query)) {
                continue;
            }
            if(strpos($haystack, trim($query), $offset) !== false) return true; // stop on first true result
        }
        return false;
    }
}
