<?php
/* 
 *  Based on some work of autoptimize plugin 
 */
defined('_JEXEC') or die;

/**
 * Class SCMinificationBase
 */
abstract class SCMinificationBase
{
    protected $content = '';
    protected $tagWarning = false;

    /**
     * SCMinificationBase constructor.
     * @param $content
     */
    public function __construct($content)
    {
        $this->content = $content;
    }

    /**
     * Reads the page and collects tags
     * @param $justhead
     * @return mixed
     */
    abstract public function read($justhead);

    //Joins and optimizes collected things
    abstract public function minify();

    //Caches the things
    abstract public function cache();

    //Returns the content
    abstract public function getcontent();

    /**
     * Converts an URL to a full path
     * @param $url
     * @return bool|mixed|string
     */
    protected function getpath($url)
    {
        if (strpos($url, '%') !== false) {
            $url = urldecode($url);
        }
        // normalize
        if (strpos($url, '//') === 0) {
            if (self::isSsl()) {
                $url = "https:" . $url;
            } else {
                $url = "http:" . $url;
            }
        }

        $domain = ((self::isSsl()) ? "https://" : "http://") . $_SERVER['HTTP_HOST'];
        //Get uri name (example : joomla) from http://localhost/joomla
        $uri_name = str_replace($domain, '', SC_JOOMLA_SITE_URL);
        $uri_name = trim($uri_name, '/');
        if (!empty($uri_name) && strpos($url, $uri_name) === false) {
            /*
             * is internal url, for example: /media/templates/test.php
             * if true return http://localhost/joomla
             * if not will get http://localhost
             */
            $domain = SC_JOOMLA_SITE_URL;
        }

        if (strpos($url, 'http') === false) {
            $url = rtrim($domain, '/') . '/' . ltrim($url,'/');
        }

        if (strpos($url, $_SERVER['HTTP_HOST']) === false) {
            //is external url, for example: http://fonts.googleapis.com/css
            return false;
        }

        // try to remove "joomla root url" from url while not minding http<>https

        $path = str_replace(SC_JOOMLA_SITE_URL, '', $url);

        $path = JPATH_BASE .'/'. ltrim($path, '/');

        return $path;
    }

    /**
     * Converts a path to an URL
     */
    protected function getUrl($path)
    {
        if (strpos($path, '%') !== false) {
            $path = urlencode($path);
        }
        if (strpos($path, $_SERVER['HTTP_HOST']) !== false) {
            return false;
        }

        if (strpos($path, JPATH_BASE) === false) {
            return false;
        }

        $url = str_replace(JPATH_BASE, SC_JOOMLA_SITE_URL, $path);

        return $url;
    }
    /**
     * needed for filter
     * @param $in
     * @return mixed|string
     */
    protected function aoGetDomain($in)
    {
        // make sure the url starts with something vaguely resembling a protocol
        if ((strpos($in, "http") !== 0) && (strpos($in, "//") !== 0)) {
            $in = "http://" . $in;
        }

        // do the actual parse_url
        $out = parse_url($in, PHP_URL_HOST);

        // fallback if parse_url does not understand the url is in fact a url
        if (empty($out)) {
            $out = $in;
        }

        return $out;
    }

    /**
     * logger
     * @param $logmsg
     * @param bool $appendHTML
     */
    protected function aoLogger($logmsg, $appendHTML = true)
    {
        if ($appendHTML) {
            $logmsg = "<!--noptimize--><!-- " . $logmsg . " --><!--/noptimize-->";
            $this->content .= $logmsg;
        } else {
            error_log("Error: " . $logmsg);
        }
    }

    /**
     * @param $iehacks_in
     * @return mixed
     */
    protected function hideIehacks($iehacks_in)
    {
        if (strpos($iehacks_in, '<!--[if') !== false) {
            $iehacks_out = preg_replace_callback(
                '#<!--\[if.*?\[endif\]-->#is',
                function ($matches) {
                    return "%%IEHACK%%".base64_encode($matches[0])."%%IEHACK%%";
                },
                $iehacks_in
            );
        } else {
            $iehacks_out = $iehacks_in;
        }
        return $iehacks_out;
    }

    /**
     * @param $iehacks_in
     * @return mixed
     */
    protected function restoreIehacks($iehacks_in)
    {
        if (strpos($iehacks_in, '%%IEHACK%%') !== false) {
            $iehacks_out = preg_replace_callback(
                '#%%IEHACK%%(.*?)%%IEHACK%%#is',
                function ($matches) {
                    return base64_decode($matches[1]);
                },
                $iehacks_in
            );
        } else {
            $iehacks_out = $iehacks_in;
        }
        return $iehacks_out;
    }

    /**
     * @param $comments_in
     * @return mixed
     */
    protected function hideComments($comments_in)
    {
        if (strpos($comments_in, '<!--') !== false) {
            $comments_out = preg_replace_callback(
                '#<!--.*?-->#is',
                function ($matches) {
                    return "%%COMMENTS%%".base64_encode($matches[0])."%%COMMENTS%%";
                },
                $comments_in
            );
        } else {
            $comments_out = $comments_in;
        }
        return $comments_out;
    }

    /**
     * @param $comments_in
     * @return mixed
     */
    protected function restoreComments($comments_in)
    {
        if (strpos($comments_in, '%%COMMENTS%%') !== false) {
            $comments_out = preg_replace_callback(
                '#%%COMMENTS%%(.*?)%%COMMENTS%%#is',
                function ($matches) {
                    return base64_decode($matches[1]);
                },
                $comments_in
            );
        } else {
            $comments_out = $comments_in;
        }
        return $comments_out;
    }

    /**
     * @param $payload
     * @param $replaceTag
     */
    protected function injectInHtml($payload, $replaceTag)
    {
        if (strpos($this->content, $replaceTag[0]) !== false) {
            if ($replaceTag[1] === "after") {
                $replaceBlock = $replaceTag[0] . $payload;
            } elseif ($replaceTag[1] === "replace") {
                $replaceBlock = $payload;
            } else {
                $replaceBlock = $payload . $replaceTag[0];
            }
            $this->content = substr_replace(
                $this->content,
                $replaceBlock,
                strpos($this->content, $replaceTag[0]),
                strlen($replaceTag[0])
            );
        } else {
            $this->content .= $payload;
            if (!$this->tagWarning) {
                $this->content .= "<!--noptimize--><!-- SpeedCache found a problem with the HTML in your Theme, tag ";
                $this->content .= $replaceTag[0] . " missing --><!--/noptimize-->";
                $this->tagWarning = true;
            }
        }
    }


    /**
     * inject already minified code in optimized JS/CSS
     * @param $in
     * @return mixed
     */
    protected function injectMinified($in)
    {
        if (strpos($in, '%%INJECTLATER%%') !== false) {
            $out = preg_replace_callback(
                '#%%INJECTLATER%%(.*?)%%INJECTLATER%%#is',
                function ($matches) {
                    $filepath    = base64_decode(strtok($matches[1], '|'));
                    $filecontent = file_get_contents($filepath);

                    // remove BOM
                    $filecontent = preg_replace('#\x{EF}\x{BB}\x{BF}#', '', $filecontent);

                    // remove comments and blank lines
                    if (substr($filepath, - 3, 3) === '.js') {
                        $filecontent = preg_replace('#^\s*\/\/.*$#Um', '', $filecontent);
                    }

                    $filecontent = preg_replace('#^\s*\/\*[^!].*\*\/\s?#Us', '', $filecontent);
                    $filecontent = preg_replace("#(^[\r\n]*|[\r\n]+)[\s\t]*[\r\n]+#", "\n", $filecontent);

                    // specific stuff for JS-files
                    if (substr($filepath, - 3, 3) === '.js') {
                        if ((substr($filecontent, - 1, 1) !== ';') && (substr($filecontent, - 1, 1) !== '}')) {
                            $filecontent .= ';';
                        }

                        if (get_option('sc_js_trycatch') === 'on') {
                            $filecontent = 'try{' . $filecontent . '}catch(e){}';
                        }
                    } elseif ((substr($filepath, - 4, 4) === '.css')) {
                        $filecontent = SCMinificationStyles::fixurls($filepath, $filecontent);
                    }

                    // return
                    return "\n" . $filecontent;
                },
                $in
            );
        } else {
            $out = $in;
        }
        return $out;
    }

    /**
     * Determines if SSL is used.
     *
     * @since 2.6.0
     * @since 4.6.0 Moved from functions.php to load.php.
     *
     * @return bool True if SSL, otherwise false.
     */
    public static function isSsl()
    {
        if (isset($_SERVER['HTTPS'])) {
            if ('on' == strtolower($_SERVER['HTTPS'])) {
                return true;
            }

            if ('1' == $_SERVER['HTTPS']) {
                return true;
            }
        } elseif (isset($_SERVER['SERVER_PORT']) && ('443' == $_SERVER['SERVER_PORT'])) {
            return true;
        }
        return false;
    }

    /**
     * Check match from content
     *
     * @param string $tag   Tag from content
     * @param array  $array List check
     *
     * @return boolean
     */
    protected function isMatch($tag, $array)
    {
        if (!empty($array)) {
            foreach ($array as $match) {
                if (empty($match)) {
                    continue;
                }
                if (strpos($tag, $match) !== false) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Get content of external script
     * @param $url
     * @return mixed
     */
    protected function getExternalData($url)
    {
        $data = false;
        if (file_exists($url)) {
            $data = file_get_contents($url);
            if(!strpos($http_response_header[0], "200")){
                return false;
            }

            if (!empty($data)) {
                return $data;
            }
        }
        // try to use CURL for URLs
        if (function_exists('curl_init')) {
            $ch = curl_init();
            $options = array(
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_HEADER => true,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_AUTOREFERER => true,
                CURLOPT_CONNECTTIMEOUT => 120,
                CURLOPT_TIMEOUT => 120,
                CURLOPT_SSL_VERIFYPEER => FALSE,
            );
            curl_setopt_array($ch, $options);
            $response = curl_exec($ch);
            // Then, after your curl_exec call:
            $header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
            $headers = substr($response, 0, $header_size);
            $data = substr($response, $header_size);
            curl_close($ch);

            $headers = (explode("\r\n", $headers));
            if (isset($headers[0]) && !strpos($headers[0], "200")) {
                return false;
            }
        }

        return $data;
    }
}
