<?php
/* 
 *  Based on some work of autoptimize plugin 
 */
defined('_JEXEC') or die;

class SCMinificationStyles extends SCMinificationBase
{
    private $css = array();
    private $csscode = array();
    private $defer = false;
    private $cssinlinesize = '';
    private $grfonts = array("fonts.googleapis.com");
    private $group_fonts = false;
    private $turn_on_css = array();
    private $group_css = false;
    private $css_min_arr = array();
    private $cssExcludes = array();
    protected $cssmatches = array();
    private $defaultExcludeCSS = array();
    private $minified_url = array();
    private $exclude_inline_css = false;
    private $current_url = '';
    /**
     * Reads the page and collects style tags
     * @param $options
     * @return bool
     */
    public function read($options)
    {
        $this->cssinlinesize = 256;

        // group css?
        $this->group_css = $options['groupcss'];
        // group font
        $this->group_fonts = $options['groupfonts'];

        // Exclude js from group speedcache 2.1
        $this->cssExcludes = array_map('trim', $options['cssExcludes']);

        // what CSS shouldn't be exclude
        $this->defaultExcludeCSS = $options['css_exclude'];

        $this->exclude_inline_css = $options['exclude_inline_css'];
        // should we defer css?
        // value: true/ false
        $this->defer = $options['defer'];
        // Get current url of page
        $this->current_url = $options['url'] . '_minifycss';
        //get dk
        $this->turn_on_css = $options['turn_on_css'];

        // exclude (no)script, as those may contain CSS which should be left as is
        if (strpos($this->content, '<script') !== false) {
            $this->content = preg_replace_callback(
                '#<noscript.*?<\/noscript>#is',
                function ($matches) {
                    return "%%SCRIPT" . SC_MINIFY_HASH . "%%" . base64_encode($matches[0]) . "%%SCRIPT%%";
                },
                $this->content
            );
        }

        // Save IE hacks
        $this->content = $this->hideIehacks($this->content);

        // hide comments
        $this->content = $this->hideComments($this->content);

        // Get <style> and <link>
        if (preg_match_all('#(<style[^>]*>.*</style>)|(<link[^>]*stylesheet[^>]*>)#Usmi', $this->content, $matches)) {
            foreach ($matches[0] as $tag) {
                if ($this->isMatch($tag, $this->defaultExcludeCSS)) {
                    continue;
                }
                if ($this->isMatch($tag, $this->cssExcludes)) {
                    //Exclude css by customer
                    continue;
                }

                // Get the media
                if (strpos($tag, 'media=') !== false) {
                    preg_match('#media=(?:"|\')([^>]*)(?:"|\')#Ui', $tag, $medias);
                    $medias = explode(',', $medias[1]);
                    $media = array();
                    foreach ($medias as $elem) {
                        if (empty($elem)) {
                            $elem = 'all';
                        }
                        $media[] = $elem;
                    }
                } else {
                    // No media specified - applies to all
                    $media = array('all');
                }

                //Check exclude style with media from group css
                if ($this->group_css) {
                    if ((count($media) === 1 && !in_array('all', $media)) || (count($media) > 1)) {
                        continue;
                    }
                }

                if (preg_match('#<link.*href=("|\')(.*)("|\')#Usmi', $tag, $source)) {
                    if ($this->group_fonts && $this->isMatch($tag, $this->grfonts)) {
                        // google font link
                        $this->css[] = array($media, 'GOOGLEFONT;'.$source[2], $tag);
                        continue;
                    }
                    // <link>
                    $url = current(explode('?', $source[2], 2));
                    $path = $this->getpath($url);

                    if ($path !== false && preg_match('#\.css$#', $path)) {
                        // Good link
                        $this->css[] = array($media, $path, $tag);
                    }
                } else {
                    // Inline css in style tags can be wrapped in comment tags, so restore comments.
                    $tag = $this->restoreComments($tag);

                    if ($this->exclude_inline_css) {
                        continue;
                    }

                    if (preg_match('#<style.*>(.*)</style>#Usmi', $tag, $code)) {
                        $regex = '#^.*<!\[CDATA\[(?:\s*\*/)?(.*)(?://|/\*)\s*?\]\]>.*$#sm';
                        $code = preg_replace($regex, '$1', $code[1]);
                        // Font check
                        $font_face = array('@font-face');
                        if (!$this->group_fonts && $this->isMatch($code, $font_face)) {
                            continue;
                        }
                        if ($this->group_css || $this->group_fonts) {
                            $this->css[] = array($media, 'INLINE;' . $code, $tag);
                        }
                    }
                }
            }
            return true;
        }
        // Really, no styles?
        return false;
    }
    /**
     * Joins and optimizes CSS
     * @return bool
     */
    public function minify()
    {
        foreach ($this->css as $k => $group) {
            list($media, $css, $tag) = $group;
            $csscode = '/*SC_CSS_NOT_CONTENT*/';
            $cssPath = $css;
            if (preg_match('#^INLINE;#', $css)) {
                // <INLINE style>
                $csscode = preg_replace('#^INLINE;#', '', $css);
            } elseif (preg_match('#^GOOGLEFONT;#', $css)) {
                $css = preg_replace('#^GOOGLEFONT;#', '', $css);
                if (strpos($css, '//') === 0) {
                    if (self::isSsl()) {
                        $http = 'https:';
                    } else {
                        $http = 'http:';
                    }
                    $css = $http . $css;
                }

                //get css from server
                $csscode = $this->getExternalData($css);

                if (!$csscode) {
                    continue;
                }

                foreach ($this->turn_on_css as $turn) {
                    if (strpos($turn, '?') !== false) {
                        $turn = substr($turn, 0, strpos($turn, '?'));
                    }
                    if (strpos($cssPath, $turn) !== false) {
                        //font
                        $csscode = '/*ALREADY_MINIFIED*/' . $csscode;
                    }
                }
            } else {
                //<link>
                if ($css !== false && file_exists($css) && is_readable($css)) {
                    $csscode = $this->fixurls($cssPath, file_get_contents($css));
                    $csscode = preg_replace('/\x{EF}\x{BB}\x{BF}/', '', $csscode);

                    foreach ($this->turn_on_css as $turn) {
                        if (strpos($turn, '?') !== false) {
                            $turn = substr($turn, 0, strpos($turn, '?'));
                        }
                        if (strpos($cssPath, $turn) !== false) {
                            $csscode = '/*ALREADY_MINIFIED*/' . $csscode;
                        }
                    }
                }
            }

            if (strpos($csscode, '@import url') !== false) {
                continue;
            }

            $this->csscode[] = array($media, $csscode, $tag);
        }

        if (!empty($this->csscode)) {
            if ($this->group_css) {
                // Ready minified css
                $media_group = array('all');
                foreach ($this->csscode as $group) {
                    list($media, $csscode, $tag) = $group;
                    //Remove all old script to group
                    if (strpos($csscode, 'SC_CSS_NOT_CONTENT') === false) {
                        // Check if can not get script content, it will be not remove
                        $this->content = str_replace($tag, '', $this->content);
                    }
                }
                $hash = md5($this->current_url);

                // Check for already-minified code
                $cacheMinify = new SCMinificationCache($hash, 'css');
                if ($cacheMinify->check()) {
                    $cssCacheExist = $cacheMinify->retrieve();

                    $this->css_min_arr[] = array($media_group, $hash, $cssCacheExist, 'GROUP_NOT_TAG');
                    return true;
                }
                unset($cacheMinify);

                // Minify
                $groupCode = '';
                foreach ($this->csscode as $group) {
                    list($media, $csscode, $tag) = $group;
                    // Minify
                    if (preg_match('#ALREADY_MINIFIED#', $csscode)) {
                        if (class_exists('Minify_CSS_Compressor')) {
                            $tmp_code = trim(Minify_CSS_Compressor::process($csscode));
                        } elseif (class_exists('CSSmin')) {
                            $cssmin = new CSSmin();
                            if (method_exists($cssmin, 'run')) {
                                $tmp_code = trim($cssmin->run($csscode));
                            } elseif (is_callable(array($cssmin, 'minify'))) {
                                $tmp_code = trim(CssMin::minify($csscode));
                            }
                        }
                        if (!empty($tmp_code)) {
                            $csscode = '/*SC_MINIFIED_CSS*/'.$tmp_code;
                            unset($tmp_code);
                        }
                    }

                    $groupCode .= '/*SC_GROUP_CSS*/'. $csscode . "\n";
                }

                $this->css_min_arr[] = array($media_group, $hash, $groupCode, 'GROUP_NOT_TAG');
            } else {
                foreach ($this->csscode as $group) {
                    list($media, $csscode, $tag) = $group;

                    $hash = md5($csscode);
                    $cacheMinify = new SCMinificationCache($hash, 'css');
                    if ($cacheMinify->check()) {
                        $cssCacheExist = $cacheMinify->retrieve();
                        $this->css_min_arr[] = array($media, $hash, $cssCacheExist, $tag);
                        continue;
                    }
                    unset($cacheMinify);

                    // Return if not in list minify
                    if (!preg_match('#ALREADY_MINIFIED#', $csscode)) {
                        continue;
                    }

                    // Minify
                    if (class_exists('Minify_CSS_Compressor')) {
                        $tmp_code = trim(Minify_CSS_Compressor::process($csscode));
                    } elseif (class_exists('CSSmin')) {
                        $cssmin = new CSSmin();
                        if (method_exists($cssmin, 'run')) {
                            $tmp_code = trim($cssmin->run($csscode));
                        } elseif (is_callable(array($cssmin, 'minify'))) {
                            $tmp_code = trim(CssMin::minify($csscode));
                        }
                    }
                    if (!empty($tmp_code)) {
                        $csscode = '/*SC_MINIFIED_CSS*/'.$tmp_code;
                        unset($tmp_code);
                    }

                    $this->css_min_arr[] = array($media, $hash, $csscode, $tag);
                }
            }
        }

        return true;
    }

    //Caches the CSS in uncompressed, deflated and gzipped form.
    public function cache()
    {
        if (!empty($this->css_min_arr)) {
            foreach ($this->css_min_arr as $group) {
                list($media, $hash, $csscode, $tag) = $group;

                $cacheMinify = new SCMinificationCache($hash, 'css');
                if (!$cacheMinify->check()) {
                    // Cache our code
                    $cacheMinify->cache($csscode, 'text/css');
                }

                $url = SC_JOOMLA_SITE_URL . $cacheMinify->getname();

                $this->minified_url[] = array($media, $hash, $url, $tag);
            }
        }
    }

    /**
     * Returns the content
     * @return mixed|string
     */
    public function getcontent()
    {
        $params = JComponentHelper::getParams('com_speedcache');

        // Inject the new stylesheets
        if ($this->defer) {
            if ($params->get('effect_defer_css', false) === 'fade') {
                $loader = '<style type="text/css">
                        body {
                            opacity: 0;
                            -webkit-transition: opacity 1s ease-in;
                               -moz-transition: opacity 1s ease-in;
                                -ms-transition: opacity 1s ease-in;
                                 -o-transition: opacity 1s ease-in;
                                    transition: opacity 1s ease-in;
                        }
                        body.sc-defer-loader {
                            opacity: 1;
                        }
                        </style>';
            } else {
                // Get pre-loading settings
                $preload_config = $params->get('preloading_settings', false);
                // Get image url
                $imgurl = SC_JOOMLA_SITE_URL . 'plugins/system/speedcache/minifications/load-wait.gif';
                if ($preload_config && isset($preload_config->preloading_icon)) {
                    $imgurl = SC_JOOMLA_SITE_URL . 'plugins/system/speedcache/minifications/preload_images/'. $preload_config->preloading_icon;
                }
                // Get style of image , set center of window
                $imgstyle = 'position: absolute;z-index: 9999;top: calc((100% - 200px)/2);left: calc((100% - 200px)/2);';
                if ($preload_config && isset($preload_config->preloading_size)) {
                    $imgstyle = 'position: absolute;z-index: 9999;';
                    $imgstyle .= 'width : '. $preload_config->preloading_size . 'px;';
                    $imgstyle .= 'height : '. $preload_config->preloading_size . 'px;';
                    $imgstyle .= 'top: calc((100% - '.$preload_config->preloading_size.'px)/2);';
                    $imgstyle .= 'left: calc((100% - '.$preload_config->preloading_size.'px)/2);';
                }
                // Create image link
                $image = '<img id="sc-loading-image" style="'.$imgstyle.'" src="'.$imgurl.'" alt="Loading..." />';
                // Get background color
                $background = '#fff';
                if ($preload_config && isset($preload_config->preloading_background)) {
                    $background = $preload_config->preloading_background;
                }
                // Get transparency
                $opacity = 1;
                if ($preload_config && isset($preload_config->preloading_transparency)) {
                    $opacity = $preload_config->preloading_transparency;
                }

                $loader = '<div id="sc-loading" 
                       style="width: 100%;height: 100%;top: 0;left: 0;position: fixed;display: block;z-index: 9999;transition: visibility 0s, opacity 0.1s linear;
                       opacity: '.$opacity.';
                       background-color: '.$background.';">'.$image.'</div>';
            }

            if (preg_match('#(<body).*(>)#Usmi', $this->content, $source)) {
                $this->injectInHtml($loader, array($source[0], 'after'));
            }
        }


        $replaceTag = array('<title', 'before');

        if ($this->defer) {
            $deferredCssBlock = "<script data-cfasync='false'>function lCss(url,media) {var d=document;var l=d.createElement('link');";
            // phpcs:ignore WordPress.WP.EnqueuedResources -- This is writing direct style to the content
            $deferredCssBlock .= "l.rel='stylesheet';l.type='text/css';l.href=url;l.media=media;aoin=d.getElementsByTagName('noscript')[0];aoin.parentNode.insertBefore(l,aoin.nextSibling);}";
            $deferredCssBlock .= 'function deferredCSS() {';
            $noScriptCssBlock = '<noscript>';
        }

        if (!empty($this->minified_url)) {
            foreach ($this->minified_url as $group) {
                list($media, $hash, $url, $tag) = $group;

                $media_ele = '';
                if (!empty($media)) {
                    $media_ele = implode(',', $media);
                }

                $cacheMinify = new SCMinificationCache($hash, 'css');
                if ($cacheMinify->check()) {
                    $csscode = $cacheMinify->retrieve();
                }

                //Add the stylesheet either deferred (import at bottom) or normal links in head
                if ($this->defer) {
                    $deferredCssBlock .= "lCss('" . $url . "','" . $media_ele . "');";
                    $noScriptCssBlock .= '<link type="text/css" media="' . $media_ele;
                    // phpcs:ignore WordPress.WP.EnqueuedResources -- This is writing direct style to the content
                    $noScriptCssBlock .= '" href="' . $url . '" rel="stylesheet" />';
                } else {
                    if (strlen($csscode) > $this->cssinlinesize) {
                        $minified = '<link type="text/css" media="' . $media_ele . '" href="';
                        // phpcs:ignore WordPress.WP.EnqueuedResources -- This is writing direct style to the content
                        $minified .= $url . '" rel="stylesheet" />';
                    } elseif (strlen($csscode) > 0) {
                        $minified = '<style type="text/css" media="' . $media_ele . '">' . $csscode . '</style>';
                    }

                    if ($this->group_css) {
                        $this->injectInHtml($minified, $replaceTag);
                    } else {
                        $this->content = str_replace($tag, $minified, $this->content);
                    }

                }
            }
        }

        if ($this->defer) {
            $deferredCssBlock .= "}if(window.addEventListener){window.addEventListener('DOMContentLoaded',deferredCSS,false);}else{window.onload = deferredCSS;}";
            $deferredCssBlock .= "window.onload = function() {";
            if ($params->get('effect_defer_css', false) === 'fade') {
                $deferredCssBlock .= "document.getElementsByTagName('body')[0].classList.add('sc-defer-loader');";
            } else {
                $deferredCssBlock .= "document.getElementById('sc-loading').style.visibility = 'hidden';";
            }
            $deferredCssBlock .= "}</script>";
            $noScriptCssBlock .= '</noscript>';
            $this->injectInHtml($noScriptCssBlock, $replaceTag);
            $this->injectInHtml($deferredCssBlock, array('</body>', 'before'));
        }

        // restore (no)script
        if (strpos($this->content, '%%SCRIPT%%') !== false) {
            $this->content = preg_replace_callback(
                '#%%SCRIPT' . SC_MINIFY_HASH . '%%(.*?)%%SCRIPT%%#is',
                function ($matches) {
                    return base64_decode($matches[1]);
                },
                $this->content
            );
        }

        // restore IE hacks
        $this->content = $this->restoreIehacks($this->content);
        // restore comments
        $this->content = $this->restoreComments($this->content);
        //Return the modified stylesheet
        return $this->content;
    }

    /**
     * @param $file
     * @param $code
     * @return mixed
     */
    public static function fixurls($file, $code)
    {
        $file = str_replace(JPATH_ROOT, '/', $file);

        $dir = dirname($file); //Like

        // quick fix for import-troubles in e.g. arras theme
        $code = preg_replace('#@import ("|\')(.+?)\.css("|\')#', '@import url("${2}.css")', $code);

        if (preg_match_all('#url\((?!data)(?!\#)(?!"\#)(.*)\)#Usi', $code, $matches)) {
            $replace = array();
            foreach ($matches[1] as $k => $url) {
                // Remove quotes
                $url = trim($url, " \t\n\r\0\x0B\"'");
                $noQurl = trim($url, "\"'");
                if ($url !== $noQurl) {
                    $removedQuotes = true;
                } else {
                    $removedQuotes = false;
                }
                $url = $noQurl;
                if (substr($url, 0, 1) == '/' || preg_match('#^(https?://|ftp://|data:)#i', $url)) {
                    //URL is absolute
                    continue;
                } else {
                    // relative URL
                    $newurl = preg_replace(
                        '/https?:/',
                        '',
                        str_replace(" ", "%20", SC_JOOMLA_SITE_URL . str_replace('//', '/', $dir . '/' . $url))
                    );

                    $hash = md5($url);
                    $code = str_replace($matches[0][$k], $hash, $code);

                    if (!empty($removedQuotes)) {
                        $replace[$hash] = 'url(\'' . $newurl . '\')';
                    } else {
                        $replace[$hash] = 'url(' . $newurl . ')';
                    }
                }
            }
            //Do the replacing here to avoid breaking URLs
            $code = str_replace(array_keys($replace), array_values($replace), $code);
        }
        return $code;
    }
}
