<?php
/**
 * Speedcache
 *
 * We developed this code with our hearts and passion.
 * We hope you found it useful, easy to understand and to customize.
 * Otherwise, please feel free to contact us at contact@joomunited.com *
 * @package Speedcache
 * @copyright Copyright (C) 2016 JoomUnited (https://www.joomunited.com). All rights reserved.
 * @license GNU General Public License version 2 or later; http://www.gnu.org/licenses/gpl-2.0.html
 */

defined('_JEXEC') or die;

/**
 * Joomla! Page Cache Plugin.
 *
 * @since  1.5
 */
class PlgSystemSpeedcache extends JPlugin
{
    protected $app = null;

    protected $sc_global = null;

    protected $user = null;

    protected $pageInfos = null;

    protected $db = null;

    protected $cache_key = null;

    protected $url = null;

    protected $cache_page_info = null;

    protected $check_url_cache = array();

    protected $username = null;

    protected $name = null;

    protected $email = null;

    protected $detect = null;

    protected $X1 = null;

    protected $body;

    protected $clean_tasks = array('save', 'apply', 'publish', 'unpublish', 'trash');

    /**
     * Constructor.
     *
     * @param   object &$subject The object to observe.
     * @param   array $config An optional associative array of configuration settings.
     *
     * @since   1.5
     */
    public function __construct(& $subject, $config)
    {
        parent::__construct($subject, $config);

        // Get the application if not done by JPlugin.
        if (!isset($this->app)) {
            $this->app = JFactory::getApplication();
        }

        //Get speedcache component parameters
        $this->sc_global = JComponentHelper::getParams('com_speedcache');

        //Get new current url
        $this->url = $this->cleanUrl(JUri::getInstance()->toString());

        //Get new current page information
        $this->pageInfos = $this->getCachesPage();

        if ($this->app->isClient('site')) {
            require_once 'Mobile-Detect-2.8.25/Mobile_Detect.php';
            $this->detect = new \Joomunited\SpeedCache\Mobile_Detect\Mobile_Detect;
        }
    }

    public function getCacheKey()
    {
        //Create the cache_key value
        $cache_key = $this->url;

        //Ignore parameter for url
        if (isset($this->pageInfos->ignoreparams) && $this->pageInfos->ignoreparams) {
            $parsed = parse_url($this->url);

            if (isset($parsed['query'])) {
                $cache_key = (empty($parsed['scheme']) ? 'http' : $parsed['scheme']) . '://'
                    . $parsed['host']
                    . (empty($parsed['port']) ? '' : ':' . $parsed['port'])
                    . (empty($parsed['path']) ? '' : $parsed['path'])
                    . (empty($parsed['fragment']) ? '' : '#' . $parsed['fragment']);

            }
        }

        if (isset($this->user->guest) && !$this->user->guest) {
            //In case user is connected check if we create a cache file per user
            if ($this->sc_global->get('distinct_user_cache', '0')) {
                $cache_key .= '_' . $this->user->id;
            } else {
                $cache_key .= '_X';
            }
        } else {
            $cache_key .= '_0';
        }

        if (!empty($this->detect)) {
            // Detect devides
            if ($this->detect->isMobile() && !$this->detect->isTablet()) {
                //  The first X will be D for Desktop cache
                //                            M for Mobile cache
                //                            T for Tablet cache
                if ($this->sc_global->get('cache_mobile', false) == 0) {
                    $this->X1 = 'D';
                    $cache_key .= '_speed_cache_desktop';
                }
                if ($this->sc_global->get('cache_mobile', false) == 1) {
                    $this->X1 = 'M';
                    $cache_key .= '_speed_cache_mobile';
                }
                if ($this->sc_global->get('cache_mobile', false) == 2) {
                    return false;
                }
            } elseif ($this->detect->isTablet()) {
                if ($this->sc_global->get('cache_tablet', false) == 0) {
                    $this->X1 = 'D';
                    $cache_key .= '_speed_cache_desktop';
                }
                if ($this->sc_global->get('cache_tablet', false) == 1) {
                    $this->X1 = 'T';
                    $cache_key .= '_speed_cache_tablet';
                }
                if ($this->sc_global->get('cache_tablet', false) == 2) {
                    return false;
                }
            } else {
                if ($this->sc_global->get('cache_desktop', false) == 0) {
                    $this->X1 = 'D';
                    $cache_key .= '_speed_cache_desktop';
                }
                if ($this->sc_global->get('cache_desktop', false) == 1) {
                    return false;
                }
            }
        }

        return $cache_key;
    }

    public function checkExcludeCache()
    {
        if (!isset($this->pageInfos)) {
            return true;
        }
        //Check if this page should be cached if not fall back to joomla's default cache
        if (!$this->pageInfos || $this->cleanUrl($this->pageInfos->url) !== $this->url) {
            return true;
        }

        if (isset($this->user->guest) && $this->user->guest) {
            if (!$this->pageInfos->guest) {
                return true;
            }
        } else {
            if (!$this->pageInfos->logged) {
                return true;
            }
        }

        return false;
    }

    public function checkRequest()
    {
        $ajaxheader = $this->app->input->server->get('HTTP_X_REQUESTED_WITH', null);
        // Is ajax request ?
        if (!empty($ajaxheader) && strtolower($ajaxheader) === 'xmlhttprequest') {
            return true;
        }

        return false;
    }

    public function onAfterInitialise()
    {
        //Do not use cache for some cases
        if ($this->app->isClient('administrator') ||
            $this->app->input->getMethod() !== 'GET' ||
            $this->app->get('offline', '0') ||
            count($this->app->getMessageQueue()) ||
            $this->checkRequest()
        ) {
            return;
        }

        //Login url with per user to preload cache
        $id = $this->app->input->server->get('HTTP_USERID', null);
        //check token preload
        $bearer = 'Bearer_' . $this->sc_global->get('preloading_token', false);
        $isPreloadRequest = $this->app->input->server->get('HTTP_AUTHORIZATION_SC', null) === $bearer;

        if (!empty($id) && $isPreloadRequest) {
            //login per user
            $instance = JFactory::getUser($id);
            $instance->set('guest', 0);

            $session = JFactory::getSession();
            $session->set('user', $instance);

            JFactory::getApplication()->checkSession();
            $db = JFactory::getDbo();
            $query = $db->getQuery(true)
                ->update($db->quoteName('#__session'))
                ->set($db->quoteName('guest') . ' = ' . $db->quote($instance->get('guest')))
                ->set($db->quoteName('username') . ' = ' . $db->quote($instance->get('username')))
                ->set($db->quoteName('userid') . ' = ' . (int)$instance->get('id'))
                ->where($db->quoteName('session_id') . ' = ' . $db->quote($session->getId()));
            $db->setQuery($query);
            $db->execute();
        }

        // Get user information
        $this->user = JFactory::getUser();

        // Return if it is preload request
        if ($isPreloadRequest) {
            return;
        }

        //Do not use cache for this case
        if ($this->checkExcludeCache()) {
            return;
        }

        $cache_key = $this->getCacheKey();
        //Do not use cache for this case
        if (!$cache_key) {
            return;
        }

        //Check if cache file exists and if it's expired
        $cacheFile = JPATH_CACHE . DIRECTORY_SEPARATOR . 'speedcache' . DIRECTORY_SEPARATOR . md5($cache_key) . '.php';

        $datas = false;
        if (file_exists($cacheFile)) {
            $modifiedDate = filemtime($cacheFile);
            if (isset($this->pageInfos->lifetime)) {
                if ($this->pageInfos->lifetime == '1') {
                    $timelife = $this->sc_global->get('cache_lifetime', 1440) * 60;
                } else {
                    $timelife = $this->pageInfos->specifictime * 60;
                }
            } else {
                $timelife = $this->sc_global->get('cache_lifetime', 1440) * 60;
            }

            //Check if file modification time exceed max cache time
            if ($modifiedDate + $timelife > time()) {
                // If the etag matches the page id
                // from Joomla 3.5 source code
                if (!headers_sent() && (isset($_SERVER['HTTP_IF_NONE_MATCH']))) {
                    //Get browser etag
                    $etag = stripslashes($_SERVER['HTTP_IF_NONE_MATCH']);
                    //Check if browser etag is same than generated etag
                    if ($etag == md5($cache_key . $modifiedDate . JSession::getFormToken()) ||
                        (isset($_SERVER['HTTP_IF_MODIFIED_SINCE']) && $modifiedDate === strtotime($_SERVER['HTTP_IF_MODIFIED_SINCE']))) {
                        //Return directly not modified header and exit
                        if ($this->sc_global->get('use_browser_cache', false)) {
                            header('HTTP/1.x 304 Not Modified', true);
                            $this->app->close();
                        }
                    }
                }

                //Load file cache content and remove security string
                $datas = file_get_contents($cacheFile);
                $datas = substr($datas, strlen('<?php die("Access Denied"); ?>'));
            }
        }

        //If we have cached datas lets process it
        if ($datas !== false) {
            if (file_exists(JPATH_PLUGINS . '/system/piwik/piwik.php')) {
                require_once JPATH_PLUGINS . '/system/piwik/piwik.php';
                if (JPluginHelper::isEnabled('system', 'piwik')) {
                    if (method_exists('PlgSystemPiwik','callPiwik')) {
                        PlgSystemPiwik::callPiwik();
                    }
                }
            }

            $unserializedDatas = unserialize($datas);
            if ($unserializedDatas === false) {
                return;
            }

            //Search for a replaced string in the cached page and replace it with the proper values.
            $token = JSession::getFormToken();
            $search = array(
                '###speedCACHE#TOKEN#!speedCACHE###',
                '###speedCACHE#EMAIL#!speedCACHE###',
                '###speedCACHE#USERNAME#!speedCACHE###',
                '###speedCACHE#NAME#!speedCACHE###'
            );
            $body = str_replace(
                $search,
                array($token, $this->user->email, $this->user->username, $this->user->name),
                $unserializedDatas['body']
            );

            //Set headers
            foreach ($unserializedDatas['headers'] as $header) {
                if ($this->sc_global->get('use_browser_cache', false) && strtolower($header['name']) === "cache-control") {
                    $header['value'] = 'no-cache';
                }
                $this->app->setHeader($header['name'], $header['value']);
            }

            //Update the etag reference
            if ($this->sc_global->get('use_browser_cache', false)) {
                $modifiedDate = time();
                if (file_exists($cacheFile)) {
                    $modifiedDate = filemtime($cacheFile);
                }

                $this->app->setHeader('ETag', md5($cache_key . $modifiedDate . JSession::getFormToken()), true);
            }

            if ($this->sc_global->get('cache_header', 0)) {
                $this->app->setHeader('Cache-Provider', 'SpeedCache,'.$this->X1 . 'E' );
            }

            // If gzip compression is enabled in configuration and the server is compliant, compress the output.
            if ($this->app->get('gzip') &&
                !ini_get('zlib.output_compression') &&
                (ini_get('output_handler') != 'ob_gzhandler')) {
                //Compress body if requested
                $supported = array(
                    'x-gzip' => 'gz',
                    'gzip' => 'gz',
                    'deflate' => 'deflate'
                );

                // Get the supported encoding.
                $encodings = array_intersect($this->app->client->encodings, array_keys($supported));

                // Check header if exist
                $header = $this->app->getHeaders();
                $check_gzip_exist = false;
                if(!empty($header)) {
                    foreach ($header as $k => $v) {
                        if ($v['name'] == 'Content-Encoding') {
                            $check_gzip_exist = true;
                        }
                    }
                }

                if (!empty($encodings) && !headers_sent() && (connection_status() === CONNECTION_NORMAL)) {
                    // Iterate through the encodings and attempt to
                    // compress the data using any found supported encodings.
                    foreach ($encodings as $encoding) {
                        if (($supported[$encoding] == 'gz') || ($supported[$encoding] == 'deflate')) {
                            // Verify that the server supports gzip compression
                            // before we attempt to gzip encode the data.
                            if (!extension_loaded('zlib') || ini_get('zlib.output_compression')) {
                                continue;
                            }

                            // Attempt to gzip encode the data with an optimal level 4.
                            $gzdata = gzencode($body, 4, ($supported[$encoding] == 'gz') ? FORCE_GZIP : FORCE_DEFLATE);

                            // If there was a problem encoding the data just try the next encoding scheme.
                            if ($gzdata === false) {
                                continue;
                            }


                            // Set the encoding headers.
                            if (!$check_gzip_exist) {
                                $this->app->setHeader('Content-Encoding', $encoding);
                            }


                            // Replace the output with the encoded data.
                            $body = $gzdata;

                            // Compression complete, let's break out of the loop.
                            break;
                        }
                    }
                }
            }

            if (!$body) {
                return;
            }

            $this->app->setBody($body);
            // Dumps HTML page.
            echo $this->app->toString();
            // Closes the application.
            $this->app->close();
        }
    }

    /**
     * Trigger on after render
     */
    public function speedcacheOnAfterRender()
    {
        $this->body = $this->app->getBody();

        //Return html after scan url on backend
        if (!empty($this->app->input->get('scbackend')) && $this->app->input->get('scbackend') == '2') {
            echo $this->body;
            while (@ob_end_flush());
            exit;
        }

        //Do not use cache for some cases
        if ($this->app->isClient('administrator') ||
            $this->app->input->getMethod() !== 'GET' ||
            $this->app->get('offline', '0') ||
            count($this->app->getMessageQueue())) {
            return;
        }

        // Ajax load module
        if ($this->sc_global->get('ajax_load_module_config', '0')) {
            // Check if it is not httml, dont need
            if (preg_match('#</html>#i', $this->body)) {
                include_once(dirname(__FILE__) . '/ajax_load_modules/ajax_load_modules.php');
                $loadModule = new SCAjaxLoadModules();
                $this->body = $loadModule->setModules($this->body);
            }
        }
        // Lazy loading features;
        if ($this->sc_global->get('lazy_loading', 0)) {
            $check_enable = $this->checkLazyLoading($this->sc_global);
            $list_exclude = $this->sc_global->get('exclude_lazy_loading', array());
            $disallow_width = $this->sc_global->get('exclude_width_lazyload', '');
            $disallow_height = $this->sc_global->get('exclude_height_lazyload', '');
            if ($check_enable) {
                include_once(dirname(__FILE__) . '/lazy_loading/lazy_loading.php');
                if (!defined('SC_SITE_URL')) {
                    define('SC_SITE_URL', JUri::base());
                }
                $lazyLoading = new SCLazyLoading($this->body, $list_exclude, array($disallow_width, $disallow_height));

                $this->body = $lazyLoading->setup();
            }
        }

        //run minify here!
        $this->body = $this->runMinifyHtml($this->body);

        // run CDN
        if ($this->sc_global->get('cdn_active', 0)) {
            include_once(dirname(__FILE__) . '/cdn_integration/cdn_rewrite.php');
            $cdn = new SCCDNRewrite();
            $this->body = $cdn->rewrite($this->body);
        }

        $this->app->setBody($this->body);
    }

    /**
     * After render.
     *
     * @return   void
     *
     * @since   1.5
     */
    public function speedcacheOnAfterRespond()
    {
        //Do not use cache for some cases
        if ($this->app->isClient('administrator') ||
            $this->app->input->getMethod() !== 'GET' ||
            $this->app->get('offline', '0') ||
            count($this->app->getMessageQueue()) ||
            $this->checkRequest()
        ) {
            return;
        }

        if ($this->checkExcludeCache()) {
            return;
        }

        $cache_key = $this->getCacheKey();
        //Do not use cache for this case
        if (!$cache_key) {
            return;
        }

        $body = $this->app->getBody();

        //Get current page full html response
        $headers = $this->app->getHeaders();
        //Decode gzipped content
        if (!empty($headers)) {
            foreach ($headers as $headerKey => $headerValue) {
                if ($headerValue['name'] === 'Content-Encoding') {
                    switch ($headerValue['value']) {
                        case 'gzip':
                        case 'x-gzip':
                        case 'deflate':
                            if (!extension_loaded('zlib')) {
                                return;
                            }
                            $decodedBody = gzdecode($body);

                            if (!$decodedBody) {
                                //If we can't decode do nothing
                                return;
                            }
                            $body = $decodedBody;

                            //Remove the encoding as it's not anymore encoded
                            unset($headers[$headerKey]);

                            break 2;
                    }
                }
            }
        }

        //Replace the token by our own value
        $token = JSession::getFormToken();
        $body = str_replace($token, '###speedCACHE#TOKEN#!speedCACHE###', $body);

        // Replace user information
        if (isset($this->user->guest) && !$this->user->guest && $this->sc_global->get('distinct_user_cache', '0')) {
            $search = array(
                '###speedCACHE#EMAIL#!speedCACHE###',
                '###speedCACHE#USERNAME#!speedCACHE###',
                '###speedCACHE#NAME#!speedCACHE###'
            );
            $body = str_replace(array($this->user->email, $this->user->username, $this->user->name), $search, $body);
        }

        $datas = serialize(array('key' => $cache_key, 'body' => $body, 'headers' => $headers));

        $cacheDirectory = JPATH_CACHE . DIRECTORY_SEPARATOR . 'speedcache';
        $cacheFile = $cacheDirectory . DIRECTORY_SEPARATOR . md5($cache_key) . '.php';

        if (!file_exists($cacheDirectory)) {
            mkdir($cacheDirectory, 0777, true);
        }
        file_put_contents($cacheFile, '<?php die("Access Denied"); ?>' . $datas);
        touch($cacheFile, time());
    }


    //Check after routin if the cache need to be cleared
    public function onAfterRoute()
    {
        //Only save on post request
        if ($this->app->input->getMethod() !== 'POST') {
            return;
        }

        //if clean on admin side is not enabled quit
        if ($this->app->isClient('administrator') && !$this->sc_global->get('clear_on_admin_tasks', 1)) {
            return;
        }

        //if clean on frontend side is not enabled quit
        if ($this->app->isClient('site') && !$this->sc_global->get('clear_on_frontend_tasks', 1)) {
            return;
        }

        $task = $this->app->input->get('task');

        if (!$task) {
            return;
        }
        $task = explode('.', $task);
        if (count($task) > 1 && !in_array($task[1], $this->clean_tasks)) {
            return;
        } elseif (count($task) === 1 && !in_array($task[0], $this->clean_tasks)) {
            return;
        }

        //This is a save task let's clear our cache
        jimport('joomla.filesystem.folder');
        $cacheFolder = JPATH_SITE . DIRECTORY_SEPARATOR . 'cache' . DIRECTORY_SEPARATOR . 'speedcache';
        if (file_exists($cacheFolder)) {
            JFolder::delete($cacheFolder);
        }
        //Clear Joomla cache is required
        if ($this->sc_global->get('clear_joomla_cache', 1)) {
            $options = array(
                'defaultgroup' => '',
                'storage' => JFactory::getConfig()->get('cache_handler', ''),
                'caching' => true,
                'cachebase' => JFactory::getConfig()->get('cache_path', JPATH_SITE . '/cache')
            );

            $cache = JCache::getInstance('', $options);
            $cache->clean();
        }

        // Clear minification cache
        // Register helper class
        JLoader::register('speedcacheHelper', JPATH_ADMINISTRATOR . '/components/com_speedcache/helpers/speedcache.php');
        speedcacheHelper::clearMinification();


        //We need to recreate the zoo cache to prevent alert message
        // when a plugin create an instance of ZOO app before we clean the cache
        if (JComponentHelper::isInstalled('com_zoo') && JComponentHelper::getComponent('com_zoo', true)->enabled) {
            JFolder::create(JPATH_SITE . DIRECTORY_SEPARATOR . 'cache' . DIRECTORY_SEPARATOR . 'com_zoo');
        }
    }

    /**
     * get url for cache or exclude cache
     */
    public function getCachesPage()
    {
        $url = substr($this->url, strlen(JUri::root()));

        $this->db = JFactory::getDbo();
        $pageInfo = new stdClass();
        $pageInfo->url = $this->url;

        $trans = array(
            '.' => '\.',
            '+' => '\+',
            '?' => '\?',
            '[' => '\[',
            '^' => '\^',
            ']' => '\]',
            '$' => '\$',
            '(' => '\(',
            ')' => '\)',
            '{' => '\{',
            '}' => '\}',
            '=' => '\=',
            '<' => '\<',
            '>' => '\>',
            '|' => '\|',
            ':' => '\:',
            '-' => '\-');

        //Retrieve exclude cache exact url
        $query = 'SELECT * FROM #__speedcache_urls WHERE type = "exclude" AND url =' . $this->db->quote($url);
        $this->db->setQuery($query);
        $excludeURL = $this->db->loadObject();
        if (!empty($excludeURL)) {
            if ($excludeURL->excludeguest) {
                $pageInfo->guest = false;
            }
            if ($excludeURL->excludelogged) {
                $pageInfo->logged = false;
            }
            if (isset($pageInfo->guest) && isset($pageInfo->logged)) {
                //turn off joomla cache if exclude cache for this url
                JFactory::getConfig()->set('caching', 0);
                return $pageInfo;
            }
        }

        //Retrieve exclude cache rules
        $query = 'SELECT * FROM #__speedcache_urls WHERE type="rules_exclude"';
        $this->db->setQuery($query);
        $excludeRules = $this->db->loadAssocList();
        if (!empty($excludeRules)) {
            foreach ($excludeRules as $v) {
                $v['url'] = strtr($v['url'], $trans);
                if (strpos($v['url'], '/*') === strlen($v['url']) - 2) {
                    // Url ends by /*
                    $v['url'] = substr($v['url'], 0, strlen($v['url']) - 2);
                    $v['url'] = $v['url'] . '/(?:$|\/?.*)';
                } else {
                    // Replace all other occurrences of *
                    $v['url'] = str_replace('*', '(?:.*)', $v['url']);
                }
                if ((@preg_match('@' . $v['url'] . '@', $this->url, $matches) > 0)) {
                    //if current url indifferent with the table, get current url
                    if (!isset($pageInfo->guest) && $v['excludeguest']) {
                        $pageInfo->guest = false;
                    }
                    if (!isset($pageInfo->logged) && $v['excludelogged']) {
                        $pageInfo->logged = false;
                    }
                }
            }
            if (isset($pageInfo->guest) && isset($pageInfo->logged)) {
                //turn off joomla cache if exclude cache for this rules
                JFactory::getConfig()->set('caching', 0);
                return $pageInfo;
            }
        }
        //Retrieve include cache exact url
        $query = 'SELECT * FROM #__speedcache_urls WHERE type = "include" AND url =' . $this->db->quote($url);
        $this->db->setQuery($query);
        $includeURL = $this->db->loadObject();
        if (!empty($includeURL)) {
            if (isset($includeURL->lifetime)) {
                $pageInfo->lifetime = $includeURL->lifetime;
            }
            if (isset($includeURL->specifictime)) {
                $pageInfo->specifictime = $includeURL->specifictime;
            }
            if (!isset($pageInfo->guest) && $includeURL->cacheguest) {
                $pageInfo->guest = true;
            }

            if (!isset($pageInfo->logged) && $includeURL->cachelogged) {
                $pageInfo->logged = true;
            }
            if (isset($pageInfo->guest) && isset($pageInfo->logged)) {
                return $pageInfo;
            }
        }

        //Retrieve include cache rules
        $query = 'SELECT * FROM #__speedcache_urls WHERE type="rules_include"';
        $this->db->setQuery($query);
        $includeRules = $this->db->loadAssocList();
        if (!empty($includeRules)) {
            foreach ($includeRules as $v) {
                $v['url'] = strtr($v['url'], $trans);
                if (strpos($v['url'], '/*') === strlen($v['url']) - 2) {
                    // Url ends by /*
                    $v['url'] = substr($v['url'], 0, strlen($v['url']) - 2);
                    $v['url'] = $v['url'] . '/(?:$|\/?.*)';
                } else {
                    // Replace all other occurrences of *
                    $v['url'] = str_replace('*', '(?:.*)', $v['url']);
                }

                if ((@preg_match('@' . $v['url'] . '@', $this->url, $matches) > 0)) {
                    if (!isset($pageInfo->lifetime)) {
                        $pageInfo->lifetime = $v['lifetime'];
                        $pageInfo->specifictime = $v['specifictime'];
                    }
                    //if current url indifferent with the table, get current url
                    if (!isset($pageInfo->guest) && $v['cacheguest']) {
                        $pageInfo->guest = true;
                    }

                    if (!isset($pageInfo->logged) && $v['cachelogged']) {
                        $pageInfo->logged = true;
                    }

                    if (isset($v['ignoreparams']) && $v['ignoreparams']) {
                        $pageInfo->ignoreparams = true;
                    }
                }
            }

            if (isset($pageInfo->guest) && isset($pageInfo->logged)) {
                return $pageInfo;
            }
        }

        if (isset($pageInfo->guest) && !isset($pageInfo->logged)) {
            $pageInfo->logged = false;
            if (!isset($pageInfo->lifetime)) {
                $pageInfo->lifetime = 1;
                $pageInfo->specifictime = 0;
            }
            //turn off joomla cache if exclude cache for this url
            JFactory::getConfig()->set('caching', 0);
            return $pageInfo;
        } elseif (!isset($pageInfo->guest) && isset($pageInfo->logged)) {
            $pageInfo->guest = false;
            if (!isset($pageInfo->lifetime)) {
                $pageInfo->lifetime = 1;
                $pageInfo->specifictime = 0;
            }
            //turn off joomla cache if exclude cache for this url
            JFactory::getConfig()->set('caching', 0);
            return $pageInfo;
        }
        return false;
    }


    /**
     * Execute minify
     * @param $content
     * @return mixed
     */
    protected function runMinifyHtml($content)
    {
        $classes = $jsTurnOn = $cssTurnOn = array();
        $jsExcludes = $cssExcludes = array();
        if ($this->app->isClient('site')) {
            define('SC_JOOMLA_SITE_URL', JUri::base());
            define('SC_JPATH_CACHE_PATH', JPATH_BASE . DIRECTORY_SEPARATOR . 'media/plg_speedcache/cache/');
            define('SC_CACHEFILE_PREFIX', 'sc_');
            define('SC_MINIFY_HASH', hash('md5', $this->url));
            include_once(dirname(__FILE__) . '/minifications/sc-minification-cache.php');
            if (SCMinificationCache::createCacheMinificationFolder()) {
                $fontMinify = $this->getTypeMinify(0);
                $cssMinify = $this->getTypeMinify(1);
                $cssMinify = array_merge($cssMinify, $fontMinify);
                $jsMinify = $this->getTypeMinify(2);
                // Load our base class
                $deferJS = $deferCSS = $cacheExternal = $excludeInlineScript = $excludeInlineStyle = false;
                include_once(dirname(__FILE__) . '/minifications/sc-minification-base.php');

                if (!empty($jsMinify) ||
                    $this->sc_global->get('minify_group_js_inhead', 0) ||
                    $this->sc_global->get('minify_group_js', 0) ||
                    $this->sc_global->get('defer_js', 0) ||
                    $this->sc_global->get('cache_external_script', 0)
                ) {
                    include_once(dirname(__FILE__) . '/minifications/sc-minification-scripts.php');
                    if (!class_exists('JSMin')) {
                        @include(dirname(__FILE__) . '/minifications/minify/minify-2.1.7-jsmin.php');
                    }
                    $classes[] = 'SCMinificationScripts';
                    foreach ($jsMinify as $v) {
                        $jsTurnOn[] = $v['file'];
                    }
                    if ($this->sc_global->get('minify_exclusion_group', 0)) {
                        $jsExcludes = $this->sc_global->get('minify_exclusion_group', 0)->js;
                    }
                    if ($this->sc_global->get('defer_js', 0)) {
                        $deferJS = true;
                    }
                }

                if (!empty($cssMinify) ||
                    $this->sc_global->get('minify_group_css', 0) ||
                    $this->sc_global->get('minify_group_fonts', 0) ||
                    $this->sc_global->get('defer_css', 0)) {
                    include_once(dirname(__FILE__) . '/minifications/sc-minification-styles.php');
                    if (!class_exists('CSSmin')) {
                        @include(dirname(__FILE__) . '/minifications/minify/yui-php-cssmin-2.4.8-4_fgo.php');
                    }
                    $classes[] = 'SCMinificationStyles';
                    foreach ($cssMinify as $v) {
                        $cssTurnOn[] = $v['file'];
                    }
                    if ($this->sc_global->get('minify_exclusion_group', 0)) {
                        $cssExcludes = $this->sc_global->get('minify_exclusion_group', 0)->css;
                    }
                    if ($this->sc_global->get('defer_css', 0)) {
                        $deferCSS = true;
                    }
                }

                $minifyHtml = false;
                if ($this->sc_global->get('minify_html', 1)) {
                    include_once(dirname(__FILE__) . '/minifications/sc-minification-html.php');
                    if (!class_exists('Minify_HTML')) {
                        @include(dirname(__FILE__) . '/minifications/minify/minify-html.php');
                    }
                    $classes[] = 'SCMinificationHtml';
                    $minifyHtml = true;
                }


                if (stripos($content, "<html") === false ||
                    stripos($content, "<html amp") !== false ||
                    stripos($content, "<html ⚡") !== false ||
                    stripos($content, "<xsl:stylesheet") !== false) {
                    return $content;
                }
                $groupCss = $groupJs = $groupFonts = $groupJsInHead = false;
                if ($this->sc_global->get('minify_group_css', 0)) {
                    $groupCss = true;
                }
                if ($this->sc_global->get('minify_group_js_inhead', 0)) {
                    $groupJsInHead = true;
                }
                if ($this->sc_global->get('minify_group_js', 0)) {
                    $groupJs = true;
                }
                if ($this->sc_global->get('minify_group_fonts', 0)) {
                    $groupFonts = true;
                }
                if ($this->sc_global->get('cache_external_script', 0)) {
                    $cacheExternal = true;
                }

                if ($this->sc_global->get('exclude_inline_script', 0)) {
                    $excludeInlineScript = true;
                }

                if ($this->sc_global->get('exclude_inline_style', 0)) {
                    $excludeInlineStyle = true;
                }

                // Set some options
                $classoptions = array(
                    'SCMinificationScripts' => array(
                        'defer' => $deferJS,
                        'groupOnlyhead' => $groupJsInHead,
                        'groupjs' => $groupJs,
                        'turn_on_js' => $jsTurnOn,
                        'jsExcludes' => $jsExcludes,
                        'cache_external' => $cacheExternal,
                        'exclude_inline_js' => $excludeInlineScript,
                        'url' => $this->url,
                    ),
                    'SCMinificationStyles' => array(
                        'defer' => $deferCSS,
                        'css_exclude' => ["admin-bar.min.css", "dashicons.min.css"],
                        'groupcss' => $groupCss,
                        'groupfonts' => $groupFonts,
                        'turn_on_css' => $cssTurnOn,
                        'cssExcludes' => $cssExcludes,
                        'exclude_inline_css' => $excludeInlineStyle,
                        'url' => $this->url,
                    ),
                    'SCMinificationHtml' => array(
                        'minifyHTML' => $minifyHtml
                    )
                );

                if (!empty($classes)) {
                    // Run the classes
                    foreach ($classes as $name) {
                        $instance = new $name($content);
                        if ($instance->read($classoptions[$name])) {
                            $instance->minify();
                            $instance->cache();
                            $content = $instance->getcontent();
                        }
                        unset($instance);
                    }
                }
            }
        }
        return $content;
    }


    /**
     * Get active minify
     * @param $type
     * @return mixed
     */
    public function getTypeMinify($type)
    {
        $db = JFactory::getDbo();
        $query = 'SELECT * FROM #__speedcache_minify_file WHERE minify=1 AND type=' . $db->quote((int)$type);
        $db->setQuery($query);
        return $db->loadAssocList();
    }


    public function speedcacheOnBeforeRender()
    {
        //Do not use cache for some cases
        if ($this->app->isClient('administrator') ||
            $this->app->input->getMethod() !== 'GET' ||
            $this->app->get('offline', '0') ||
            count($this->app->getMessageQueue())) {
            return;
        }

        if ($this->sc_global->get('ajax_load_module_config', '0')) {
            $loader_link = 'plugins/system/speedcache/ajax_load_modules/image/modajaxloader.gif';
            $inlinescript = "
                var speedcache_base_url = '" . JURI::base() . "';
                var loader_link = '".$loader_link."';
            ";
            $script = JUri::base(true) . '/plugins/system/speedcache/ajax_load_modules/js/modajaxloader.js';
            JFactory::getDocument()->addScript($script);
            JFactory::getDocument()->addScriptDeclaration($inlinescript);
        }

        // Lazy loading features;
        if ($this->sc_global->get('lazy_loading', 0)) {
            JHtml::_('jquery.framework');
            $script = JUri::base(true) . '/plugins/system/speedcache/lazy_loading/js/jquery.lazyload.min.js';
            $styles = JUri::base(true) . '/plugins/system/speedcache/lazy_loading/css/jquery.lazyload.fadein.css';
            JFactory::getDocument()->addScript($script);
            JFactory::getDocument()->addStyleSheet($styles);
        }
    }

    /**
     * Clear cache
     */
    public function speedcacheOnClearCache()
    {
        //if clean on admin side is not enabled quit
        if ($this->app->isClient('administrator') && !$this->sc_global->get('clear_on_admin_tasks', 1)) {
            return;
        }

        //if clean on frontend side is not enabled quit
        if ($this->app->isClient('site') && !$this->sc_global->get('clear_on_frontend_tasks', 1)) {
            return;
        }

        JModelLegacy::addIncludePath(JPATH_ROOT . '/administrator/components/com_cache/models');
        $model = JModelLegacy::getInstance('cache', 'CacheModel');

        if (!empty($model)) {
            $mCache = $model->getCache(0);

            foreach ($mCache->getAll() as $cache) {
                $mCache->clean($cache->group);
            }
        }

        // Clear minification cache
        // Register helper class
        JLoader::register('speedcacheHelper', JPATH_ADMINISTRATOR . '/components/com_speedcache/helpers/speedcache.php');
        speedcacheHelper::clearMinification();
    }

    /**
     * Check lazy loading enable with page
     *
     * @param object $params List settings
     *
     * @return boolean
     */
    private function checkLazyLoading($params)
    {
        $domain = (((!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off') ||
                $_SERVER['SERVER_PORT'] === 443) ? 'https://' : 'http://') . $_SERVER['HTTP_HOST'];
        //decode url with russian language
        $current_url = $domain . rawurldecode($_SERVER['REQUEST_URI']);
        $list_exclude = $params->get('exclude_lazy_loading', array());
        $list_include = $params->get('include_lazy_loading', array());
        // Check Exclude lazyloading
        if (!empty($list_exclude)) {
            $result = $this->checkRules($list_exclude, $current_url, 'exclude');
            if ($result !== 'continue') {
                return $result;
            }
        }

        // Check enable lazyloading with include
        if (!empty($list_include)) {
            if (!array_filter($list_include)) {
                // If nothing is set, enable lazyload for all pages
                return true;
            }
            $result = $this->checkRules($list_include, $current_url, false);
            if ($result !== 'continue') {
                return $result;
            }
        } else {
            // If nothing is set, enable lazyload for all pages
            return true;
        }

        return false;
    }

    /**
     * Check lazy loading enable with page
     *
     * @param array  $lists       List settings
     * @param string $current_url Current url
     * @param string $type        Type check
     *
     * @return boolean|string
     */
    public function checkRules($lists, $current_url, $type)
    {
        $check = true;
        if (!empty($type) && $type === 'exclude') {
            $check = false;
        }

        if (!empty($lists)) {
            foreach ($lists as $v) {
                if (empty($v)) {
                    continue;
                }
                // Clear blank character
                $v = trim($v);
                if (preg_match('/(\/?\&?\(\.?\*\)|\/\*|\*)$/', $v, $matches)) {
                    // End of rules is /*, /(*) , /(.*)
                    $pattent = substr($v, 0, strpos($v, $matches[0]));
                    if ($v[0] === '/') {
                        // A path of exclude url with regex
                        if ((preg_match('@' . $pattent . '@', $current_url, $matches) > 0)) {
                            return $check;
                        }
                    } else {
                        // Full exclude url with regex
                        if (!empty($pattent)) {
                            if (strpos($current_url, $pattent) !== false) {
                                return $check;
                            }
                        }
                    }
                } else {
                    if ($v[0] === '/') {
                        // A path of exclude
                        if ((preg_match('@' . $v . '@', $current_url, $matches) > 0)) {
                            return $check;
                        }
                    } else {
                        // Whole path
                        if ($v === $current_url) {
                            return $check;
                        }
                    }
                }
            }
        }
        return 'continue';
    }

    /**
     * Url to purge string
     * @param $url
     * @return string
     */
    protected function cleanUrl($url)
    {
        return trim($url, "/ \t\n\r\0\x0B");
    }
}
